#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <getopt.h>

static const char *progname;

static const char *preamble="{\n  ";
static const char *terminator="}\n";
static const char *separator=",";
static const char *modulator="\n  ";
static const char *format="0x%2.2x";
static int modulo=16;
static int skip=0;
static int rotatebyte=0;

#define OPTIONS "+p:t:s:m:z:f:o:b:r"
#define USAGE   \
  "      [-p <prefix>='{\\n  ']\n"\
  "      [-t <terminator>='}\\n' ]\n"\
  "      [-s <separator>=',' ]\n"\
  "      [-m <modulator>='\\n  ' ]\n"\
  "      [-z <modulo>=16 ]\n"\
  "      [-f <format>='0x%2.2x' ]\n"\
  "      [-o <outfile>=<stdout> ]\n"\
  "      [-b <skipbytes>=0 ]\n"\
  "      [-r] - rotatebyte\n"\
  "      [<infile>]"

static int rotate8(int value) {
int result=0;
int i;
  for(i=0;i<8;i++) {
    if(value&(1<<i)) result|=(0x80>>i);
  }
  return result;
}

int main(int argc,char *argv[]) {
int pos=0;
int c;
int option;
  progname=argv[0];
  while((option=getopt(argc,argv,OPTIONS))!=-1) {
    switch(option) {
      case 'p':
        preamble=optarg;
        break;
      case 't':
        terminator=optarg;
        break;
      case 's':
        separator=optarg;
        break;
      case 'm':
        modulator=optarg;
        break;
      case 'z':
        if(!sscanf(optarg,"%i",&modulo) || modulo<0) goto error;
        break;
      case 'b':
        if(!sscanf(optarg,"%i",&skip) || skip<0) goto error;
        break;
      case 'f':
        format=optarg;
        break;
      case 'r':
        rotatebyte=1;
        break;
      case 'o':
        if(!freopen(optarg,"w",stdout)) {
          fprintf(stderr,"%s:Error opening outfile: '%s' %s(%d)\n",progname,optarg,strerror(errno),errno);
          exit(1);
        }
        break;
      default:
      error:
        fprintf(stderr,"%s: usage %s\n",progname,USAGE); 
        exit(1);
    }
  }
  argc-=optind;
  argv+=optind;
  switch(argc) {
    case 0:
      break;
    case 1:
      if(!freopen(argv[0],"r",stdin)) {
        fprintf(stderr,"%s:Error opening infile: '%s' %s(%d)\n",progname,argv[0],strerror(errno),errno);
        exit(1);
      }
      break;
    default:
      goto error;
  }
  printf("%s",preamble);
  while((c=fgetc(stdin))!=EOF) {
    if(skip) skip--; else {
      if(pos) {
        printf("%s",separator);
        if(!(pos%modulo)) printf("%s",modulator);
      }
      if(rotatebyte) c=rotate8(c);
      printf(format,c);
      pos++;
    }
  }
  printf("%s",terminator);
  return 0;
}
